/*////////////////////////////////////////////////////////////////////////////////

  Xilinx, Inc. 2006                 www.xilinx.com

  DCM Spread Spectrum Reference Design

//////////////////////////////////////////////////////////////////////////////

  File name :       dcm_clkgen_softspread.v

  Description :Creates triangular soft spread spectrum solution using DCM_CLKGEN
					with a state machine for controlling the programming port.
  
  Date - revision : Sep 1, 2009 - v 1.0

  Version 1.0: Jim T, 		09_01_09
  Version 0.9: Alireza K	07_21_06 


  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
              provided to you "as is". Xilinx and its licensors make and you
              receive no warranties or conditions, express, implied,
              statutory or otherwise, and Xilinx specifically disclaims any
              implied warranties of merchantability, non-infringement,or
              fitness for a particular purpose. Xilinx does not warrant that
              the functions contained in these designs will meet your
              requirements, or that the operation of these designs will be
              uninterrupted or error free, or that defects in the Designs
              will be corrected. Furthermore, Xilinx does not warrantor
              make any representations regarding use or the results of the
              use of the designs in terms of correctness, accuracy,
              reliability, or otherwise.

              LIMITATION OF LIABILITY. In no event will Xilinx or its
              licensors be liable for any loss of data, lost profits,cost
              or procurement of substitute goods or services, or for any
              special, incidental, consequential, or indirect damages
              arising from the use or operation of the designs or
              accompanying documentation, however caused and on any theory
              of liability. This limitation will apply even if Xilinx
              has been advised of the possibility of such damage. This
              limitation shall apply not-withstanding the failure of the
              essential purpose of any limited remedies herein.

  Copyright  2006 Xilinx, Inc.
  All rights reserved

	Modules: SStop  - State machine control DCM_CLKGEN programming ports
				SSCONTROL - Sub-block of SStop used to serialize programming data for DCM_CLKGEN
	Purpose: Controls DFS to hop between 2 frequencies to create spread spectrum clocking 
	USAGE:
	SSmode: 00: Freq align mode, 11: SS mode for measuring modulation frequency, 10: SS mode with toggling lock, 01: reserved
	modfreq OUTPUT is the modulation frequency
	Spread: Controlled by FAST/SLOW registers
	

//////////////////////////////////////////////////////////////////////////////*/


`timescale 1ps / 1ps
module sscontrol (CLK, HOPCTL, HOPCMD, HOPRDY, DFS_LOCKED, NEXTFREQ, FREQSEL, READYFORNEXT, GRESET);
		input CLK, HOPRDY, NEXTFREQ; 
		input [1:0] FREQSEL;
		input DFS_LOCKED, GRESET;  
		output HOPCTL, HOPCMD, READYFORNEXT; 
		
		reg	HOPCTL, HOPCMD;
		reg	READYFORNEXT;
		reg NEXTBIT;
		reg [2:0] SS_STATE;
		reg [3:0] CHGMDCNTR;
		reg [7:0] SENDVAL;
		
		wire DCHANGE, MCHANGE; 
		
		parameter [2:0] 	INIT=3'b000, LOADM=3'b001, LOADD=3'b011, 
					JUMP0=3'b010, JUMP1=3'b110, JUMP2=3'b100;               
		parameter integer CLKFX_DIVIDE_IN = 2;		
		parameter integer CLKFX_MULTIPLY_IN  = 7;	
							 
		reg NEXTFREQMETA, NEXTFREQSYNC;
		reg DFS_LOCKEDMETA, DFS_LOCKEDSYNC;
		
		always @(posedge CLK or posedge GRESET)
			if (GRESET) begin
				NEXTFREQMETA <= 0; NEXTFREQSYNC <= 0;
				DFS_LOCKEDMETA <= 0; DFS_LOCKEDSYNC <= 0;
				CHGMDCNTR <= 0; READYFORNEXT <= 0; SS_STATE <= INIT; 
				HOPCMD <= 0; HOPCTL <= 0; 
			end
			else begin
				NEXTFREQMETA <= NEXTFREQ;
				NEXTFREQSYNC <= NEXTFREQMETA;
				DFS_LOCKEDMETA <= DFS_LOCKED; 
				DFS_LOCKEDSYNC <= DFS_LOCKEDMETA;
				case ( SS_STATE )    
				INIT: 							// wait for the 1st lock
					if ( !(DFS_LOCKEDSYNC && HOPRDY) ) SS_STATE <= INIT;
					else SS_STATE <= JUMP0;			
				JUMP0:  begin					// wait for request
					READYFORNEXT <= DFS_LOCKEDSYNC;
						if (NEXTFREQSYNC ) begin
							if (FREQSEL == 2'b11) SS_STATE <= LOADM;
							else SS_STATE <= LOADD;
						end
					else SS_STATE <= JUMP0;
					end
				LOADM: 
					if (CHGMDCNTR < 4'ha) begin			
						HOPCTL <= #100 1'b1;
						HOPCMD <= #100 NEXTBIT;
						CHGMDCNTR <= CHGMDCNTR + 1;
						end
					else begin
						HOPCTL <= #100 1'b0;
						HOPCMD <= #100 1'b0;
						CHGMDCNTR <= 0;
						SS_STATE <= LOADD;
					end
				LOADD: 	
					if (CHGMDCNTR < 4'ha) begin
						HOPCTL <= #100 1'b1;
						HOPCMD <= #100 NEXTBIT;
						CHGMDCNTR <= CHGMDCNTR + 1;
						end
					else begin
						HOPCTL <= #100 1'b0;
						HOPCMD <= #100 1'b0;
						CHGMDCNTR <= 0;
						SS_STATE <= JUMP1; 
					end

				JUMP1: begin 	    			// start the freq hop
					HOPCTL <= #100 1'b1;
					HOPCMD <= 1'b0;
					SS_STATE <= JUMP2;
				end	
				JUMP2: begin
					HOPCTL <= #100 1'b0;
					READYFORNEXT <= DFS_LOCKEDSYNC;			
					if (!HOPRDY || !NEXTFREQSYNC) SS_STATE <= JUMP2;
					else SS_STATE <= LOADD;			
					end	
				default: SS_STATE <= 3'bxxx;
				endcase
			end
			

		//Calculation of what value to send on M and D
		wire [7:0] D1st = 8'h46;		//Set to CLKFX_DIVIDE_IN * 35 which is LCD for 7/2
		wire [7:0] M1st = 8'hf5;		//Set to CLKFX_MULTIPLY_IN * 35 which is LCD for 7/2
		wire [7:0] M2nd = 8'h1b;
		wire [7:0] D2nd = CLKFX_DIVIDE_IN + 8'b1; 
		wire [7:0] M3rd = 8'h1d;
		wire [7:0] D3rd = CLKFX_DIVIDE_IN - 8'b1; 
		wire [7:0] M4th = 8'h1c;
		wire [7:0] D4th = CLKFX_DIVIDE_IN - 8'b1; 
		
		always @(FREQSEL or D4th or D2nd or D3rd or D1st or MCHANGE or M1st) 
			case (FREQSEL) 
			2'b00: SENDVAL <= D4th - 1;
			2'b01: SENDVAL <= D2nd - 1;
			2'b10: SENDVAL <= D3rd - 1;
			2'b11: SENDVAL <= (MCHANGE? M1st-1 : D1st-1);
			endcase

		assign MCHANGE = (SS_STATE==LOADM);
		assign DCHANGE = (SS_STATE==LOADD);
		always @(SENDVAL or CHGMDCNTR or MCHANGE or DCHANGE) begin
			case (CHGMDCNTR)
			4'h0:	NEXTBIT <=  #400 1'b1;
			4'h1:	if (MCHANGE) NEXTBIT <=  #400 1'b1;
				else if (DCHANGE) NEXTBIT <=  #400 1'b0;
				else NEXTBIT <=  #400 1'bx;
			4'h2:	NEXTBIT <= #400 SENDVAL[0];
			4'h3:	NEXTBIT <= #400 SENDVAL[1];
			4'h4:	NEXTBIT <= #400 SENDVAL[2];
			4'h5:	NEXTBIT <= #400 SENDVAL[3];
			4'h6:	NEXTBIT <= #400 SENDVAL[4];
			4'h7:	NEXTBIT <= #400 SENDVAL[5];
			4'h8:	NEXTBIT <= #400 SENDVAL[6];
			4'h9:	NEXTBIT <= #400 SENDVAL[7];
			default: NEXTBIT <= #400 1'b0;
			endcase
		end

endmodule

